const path = require("path")
const EventEmitter = require('events');

// JSDoc and another things heavily inspired by discord.js library.

/**
 * The main class to create a HTMLServerCreator server.
 * @extends {EventEmitter}
 */
class HTMLServer extends EventEmitter {
    /**
     * @param {ExpressApp} [expressApp] Application created with the Express Module
     * @param {ServerOptions} [options={}] Options of server
     * @example
     * var creator = require("htmlservercreator")
     * var app = require("express")()
     * var server = new creator.server(app, {port: 3000})
     */
    constructor(expressApp, options={}) {
        super()
        if(typeof options !== "object") throw Error("Options should be an object.")
        this.port = options.port || process.env.PORT || 3000
        this.app = expressApp
        this.emitrequestwhen404 = options.emitrequestwhen404 || false
        }
    /**
     * Listen for connections in your server
     * @param {Boolean} [defaultPort=true] Specify if you want to use the port in settings or use another.
     * @param {Number} [port=this.port] Port to listen
     * @returns {Promise<number>}
     * @example
     * server.listen(true).then(port => {
     *  console.log("Listening in port: " + port)
     * })
     */
    listen(defaultPort=true, port=this.port) {
        return new Promise((res, rej) => {
            let oof = defaultPort ? this.port : port
            this.app.listen(oof, () => {
                res(oof)
            })
        })
    }

    /**
     * Serve an html file to server
     * @param {Route} [route="/"] Route where the html file will be served
     * @param {RouteToHtmlFile} [htmlfileroute] Route of the html file in your computer
     * @emits {RequestEvent} [request]
     * @example
     * server.html("/", require("path").join(__dirname + "/index.html")) // localhost:port
     */
    html(route="/", htmlfileroute) {
        this.app.get(route, (req, res) => {
            res.sendFile(htmlfileroute)
            this.emit("request", route, req, res)
        })
    }

    /**
     * Define a html file to use when 404'd
     * @param {RouteToHtmlFile} [htmlfileroute] Route of the html file in your computer
     * @example
     * server.notfound(require("path").join(__dirname + "/404.html"))
     */
    notfound(htmlfileroute) {
        this.app.use((req, res) => {
            res.sendFile(htmlfileroute)
            this.emitrequestwhen404 ? this.emit("request", null, req, res) : undefined
        })
    }
}

module.exports = HTMLServer;

/**
 * Emitted when a request is sent to the server
 * @event {request}
 * @param {Route} [route] The route of the request
 * @param {Request} [req] Request Object
 * @param {Response} [res] Response Object
 */